@@ -11,6 +11,8 @@ from sklearn.utils.validation import check_is_fitted, check_X_y, column_or_1d, c
 from sklearn.exceptions import NotFittedError
 import random
 
+# TODO: add model in exp and define by thr CRP and the regression pb
+
 class ACXplainer:
     def __init__(
             self,
@@ -227,7 +229,7 @@ class ACXplainer:
             self.ACXplainer = BaseAgnosTree(self.model, self.d)
             self.check_is_explain = True
 
-    def compute_sdp_rf(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_sdp_rf(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the Same Decision Probability (SDP) of a set of samples X given subset S using the consistent estimator
          (Projected Forest + Quantile Regression), see Paper [ref]
@@ -262,6 +264,8 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
 
         sdp = cyext_acv.compute_sdp_rf(X, y, data, y_data, S, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
@@ -269,7 +273,7 @@ class ACXplainer:
                                        self.classifier, t)
         return sdp
 
-    def compute_sdp_rule(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_sdp_rule(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the local rule-based explanations of a set of samples X given Sufficient Explanations S using
          the consistent estimator (Projected Forest + Quantile Regression), see Paper [ref]. For each observation x, all
@@ -306,6 +310,8 @@ class ACXplainer:
 
         self.check_is_explainer()
 
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         sdp, rules = cyext_acv.compute_sdp_rule(X, y, data, y_data, S, self.ACXplainer.features,
                                                 self.ACXplainer.thresholds,
                                                 self.ACXplainer.children_left,
@@ -313,7 +319,7 @@ class ACXplainer:
                                                 min_node_size, self.classifier, t)
         return sdp, rules
 
-    def compute_sdp_maxrules(self, X, y, data, y_data, S, min_node_size=5, t=20, pi_level=0.95,
+    def compute_sdp_maxrules(self, X, y, data, y_data, S, min_node_size=5, t=None, pi_level=0.95,
                              verbose=False, algo2=False, rdm_size=20):
         """
          Estimate the maximal rule-based explanations of a set of samples X given Sufficient Explanations S using the consistent estimator
@@ -355,6 +361,9 @@ class ACXplainer:
 
         self.check_is_explainer()
 
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+
         if not verbose:
             sdp, rules, sdp_all, rules_data, w = cyext_acv.compute_sdp_maxrule(X, y, data, y_data, S,
                                                                                self.ACXplainer.features,
@@ -391,7 +400,7 @@ class ACXplainer:
 
         return sdp, rules, sdp_all, rules_data, w
 
-    def compute_ddp_rf(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_ddp_rf(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the Same Decision Probability (SDP) of a set of samples X given subset S using the consistent estimator
          (Projected Forest + Quantile Regression), see Paper [ref]
@@ -426,14 +435,15 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         ddp = cyext_acv.compute_ddp_rf(X, y, data, y_data, S, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
                                        self.classifier, t)
         return ddp
 
-    def compute_ddp_rule(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_ddp_rule(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the local rule-based explanations of a set of samples X given Sufficient Explanations S using
          the consistent estimator (Projected Forest + Quantile Regression), see Paper [ref]. For each observation x, all
@@ -469,7 +479,8 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         ddp, rules = cyext_acv.compute_ddp_rule(X, y, data, y_data, S, self.ACXplainer.features,
                                                 self.ACXplainer.thresholds,
                                                 self.ACXplainer.children_left,
@@ -477,7 +488,7 @@ class ACXplainer:
                                                 min_node_size, self.classifier, t)
         return ddp, rules
 
-    def compute_ddp_weights(self, X, y, data, y_data, S, min_node_size=5, t=20, pi_level=0.95,
+    def compute_ddp_weights(self, X, y, data, y_data, S, min_node_size=5, t=None, pi_level=0.95,
                              verbose=False, algo2=False, rdm_size=20):
         """
          Estimate the maximal rule-based explanations of a set of samples X given Sufficient Explanations S using the consistent estimator
@@ -518,7 +529,8 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         if not verbose:
             ddp, w = cyext_acv.compute_ddp_weights(X, y, data, y_data, S,
                                                                                self.ACXplainer.features,
@@ -541,7 +553,214 @@ class ACXplainer:
         return ddp, w
 
 
-    def compute_ddp_cond_weights(self, X, y, data, y_data, S, cond, min_node_size=5, t=20, pi_level=0.95,
+    def compute_cdp_cond_weights(self, X, y, down, up, data, y_data, S, cond, min_node_size=5, t=None, pi_level=0.95,
+                             verbose=False, algo2=False, rdm_size=20):
+        """
+         Compute P(Y \neq y | X_{\bar{S}} in rule_{\bar{S}}, X_S=x_S)
+        Args:
+            X (numpy.ndarray): A matrix of samples (# samples X # features) on which to compute the SDP
+
+            y (numpy.ndarray): 1-D array (# samples) the targets of X
+
+            data (numpy.ndarray): The background dataset that is used for the estimation of the SDP. It should be the
+                                training samples.
+
+            y_data (numpy.ndarray): The targets of the background dataset
+
+            S (list[list[int]]): A list that contains the indices of the columns of the variable of the Sufficient
+                                Explanations for each observation
+
+            min_node_size (int): The minimal node size of the Projected Forest
+
+            t (double): The level of variations around the prediction that defines the SDP in regression (only for regression)
+
+            pi_level (double): The minimal level of SDP for the Sufficient Explanations
+
+        Returns:
+            sdp (numpy.ndarray): A 1-D matrix (# samples), sdp[i] contains the Same Decision Probability (SDP)
+                                 of the Sufficient Explanation of observation i
+            rules (numpy.ndarray): A matrix (# samples x # features x 2), rules[i] that contains the maximal local rule
+                                   of observation i
+        """
+        X, y = check_X_y(X, y, dtype=np.double)
+        data, y_data = check_X_y(data, y_data, dtype=np.double)
+        y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
+        try:
+            check_consistent_length(X, S)
+        except ValueError as exp:
+            raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+        self.check_is_explainer()
+
+
+        ddp, w = cyext_acv.compute_cdp_cond_weights(X, y, down, up, data, y_data, S,
+                                                                           self.ACXplainer.features,
+                                                                           self.ACXplainer.thresholds,
+                                                                           self.ACXplainer.children_left,
+                                                                           self.ACXplainer.children_right,
+                                                                           cond,
+                                                                           self.ACXplainer.max_depth,
+                                                                           min_node_size, self.classifier, t,
+                                                                           pi_level)
+
+        return ddp, w
+
+
+    def compute_cdp_rf(self, X, y, down, up, data, y_data, S, min_node_size=5, t=None):
+        """
+         Estimate the Same Decision Probability (SDP) of a set of samples X given subset S using the consistent estimator
+         (Projected Forest + Quantile Regression), see Paper [ref]
+
+        Args:
+            X (numpy.ndarray): A matrix of samples (# samples X # features) on which to compute the SDP
+
+            y (numpy.ndarray): 1-D array (# samples) the targets of X
+
+            data (numpy.ndarray): The background dataset that is used for the estimation of the SDP. It should be the
+                                training samples.
+
+            y_data (numpy.ndarray): The targets of the background dataset
+
+            S (list[list[int]]): A list that contains the column indices of the variable on which we condition to compute the SDP
+                                 for each observation
+
+            min_node_size (int): The minimal node size of the Projected Random Forest
+
+            t (float):  The level of variations around the prediction that defines the SDP in regression (only for regression)
+
+        Returns:
+            sdp (numpy.ndarray):  A 1-D matrix (# samples), sdp[i] contains the Same Decision Probability (SDP)
+                                 of the Minimal Sufficient Explanation of observation i
+        """
+        X, y = check_X_y(X, y, dtype=np.double)
+        data, y_data = check_X_y(data, y_data, dtype=np.double)
+        y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
+        try:
+            check_consistent_length(X, S)
+        except ValueError as exp:
+            raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
+
+        self.check_is_explainer()
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+        ddp = cyext_acv.compute_cdp_rf(X, y, down, up, data, y_data, S, self.ACXplainer.features, self.ACXplainer.thresholds,
+                                       self.ACXplainer.children_left,
+                                       self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
+                                       self.classifier, t)
+        return ddp
+
+    def compute_cdp_rule(self, X, y, down, up, data, y_data, S, min_node_size=5, t=None):
+        """
+         Estimate the local rule-based explanations of a set of samples X given Sufficient Explanations S using
+         the consistent estimator (Projected Forest + Quantile Regression), see Paper [ref]. For each observation x, all
+         observations that falls in the rule of x has the same SDP as x.
+
+        Args:
+            X (numpy.ndarray): A matrix of samples (# samples X # features) on which to compute the SDP
+
+            y (numpy.ndarray): 1-D array (# samples) the targets of X
+
+            data (numpy.ndarray): The background dataset that is used for the estimation of the SDP. It should be the
+                                training samples.
+
+            y_data (numpy.ndarray): The targets of the background dataset
+
+            S (list[list[int]]): A list that contains the indices of the columns of the Sufficient Explanation for each observation
+
+            min_node_size (int): The minimal node size of the Projected Random forest
+
+            t (float): The level of variations around the prediction that defines the SDP in regression (only for regression)
+
+        Returns:
+            sdp (numpy.ndarray): A 1-D matrix (# samples), sdp[i] contains the Same Decision Probability (SDP)
+                                 of the Sufficient Explanation of observation i
+            rules (numpy.ndarray): A matrix (# samples x # features x 2), rules[i] that contains the local rule of observation i
+        """
+        X, y = check_X_y(X, y, dtype=[np.double, np.double])
+        data, y_data = check_X_y(data, y_data, dtype=[np.double, np.double])
+        y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
+        try:
+            check_consistent_length(X, S)
+        except ValueError as exp:
+            raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
+
+        self.check_is_explainer()
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+        ddp, rules = cyext_acv.compute_cdp_rule(X, y, down, up, data, y_data, S, self.ACXplainer.features,
+                                                self.ACXplainer.thresholds,
+                                                self.ACXplainer.children_left,
+                                                self.ACXplainer.children_right, self.ACXplainer.max_depth,
+                                                min_node_size, self.classifier, t)
+        return ddp, rules
+
+    def compute_cdp_weights(self, X, y, down, up, data, y_data, S, min_node_size=5, t=None, pi_level=0.95,
+                             verbose=False, algo2=False, rdm_size=20):
+        """
+         Estimate the maximal rule-based explanations of a set of samples X given Sufficient Explanations S using the consistent estimator
+         (Projected Forest + Quantile Regression), see Paper [ref]. For each instance x, all
+         observations that falls in the rule of x has its SDP given S above \pi as x.
+
+        Args:
+            X (numpy.ndarray): A matrix of samples (# samples X # features) on which to compute the SDP
+
+            y (numpy.ndarray): 1-D array (# samples) the targets of X
+
+            data (numpy.ndarray): The background dataset that is used for the estimation of the SDP. It should be the
+                                training samples.
+
+            y_data (numpy.ndarray): The targets of the background dataset
+
+            S (list[list[int]]): A list that contains the indices of the columns of the variable of the Sufficient
+                                Explanations for each observation
+
+            min_node_size (int): The minimal node size of the Projected Forest
+
+            t (double): The level of variations around the prediction that defines the SDP in regression (only for regression)
+
+            pi_level (double): The minimal level of SDP for the Sufficient Explanations
+
+        Returns:
+            sdp (numpy.ndarray): A 1-D matrix (# samples), sdp[i] contains the Same Decision Probability (SDP)
+                                 of the Sufficient Explanation of observation i
+            rules (numpy.ndarray): A matrix (# samples x # features x 2), rules[i] that contains the maximal local rule
+                                   of observation i
+        """
+        X, y = check_X_y(X, y, dtype=np.double)
+        data, y_data = check_X_y(data, y_data, dtype=np.double)
+        y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
+        try:
+            check_consistent_length(X, S)
+        except ValueError as exp:
+            raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
+
+        self.check_is_explainer()
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+        if not verbose:
+            ddp, w = cyext_acv.compute_cdp_weights(X, y, down, up, data, y_data, S,
+                                                                               self.ACXplainer.features,
+                                                                               self.ACXplainer.thresholds,
+                                                                               self.ACXplainer.children_left,
+                                                                               self.ACXplainer.children_right,
+                                                                               self.ACXplainer.max_depth,
+                                                                               min_node_size, self.classifier, t,
+                                                                               pi_level)
+        else:
+            ddp, w = cyext_acv.compute_cdp_weights_verbose(X, y, down, up,  data, y_data, S,
+                                                   self.ACXplainer.features,
+                                                   self.ACXplainer.thresholds,
+                                                   self.ACXplainer.children_left,
+                                                   self.ACXplainer.children_right,
+                                                   self.ACXplainer.max_depth,
+                                                   min_node_size, self.classifier, t,
+                                                   pi_level)
+
+        return ddp, w
+
+
+    def compute_ddp_cond_weights(self, X, y, data, y_data, S, cond, min_node_size=5, t=None, pi_level=0.95,
                              verbose=False, algo2=False, rdm_size=20):
         """
          Compute P(Y \neq y | X_{\bar{S}} in rule_{\bar{S}}, X_S=x_S)
@@ -579,8 +798,8 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         ddp, w = cyext_acv.compute_ddp_cond_weights(X, y, data, y_data, S,
                                                                            self.ACXplainer.features,
                                                                            self.ACXplainer.thresholds,
@@ -593,7 +812,7 @@ class ACXplainer:
 
         return ddp, w
 
-    def importance_sdp_rf(self, X, y, data, y_data, min_node_size=5, t=20,
+    def importance_sdp_rf(self, X, y, data, y_data, min_node_size=5, t=None,
                           C=[[]], pi_level=0.9, minimal=1, stop=True):
         """
          Estimate the Minimal Sufficient Explanations and the Global sdp importance of a set of samples X using the
@@ -646,6 +865,7 @@ class ACXplainer:
         y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
         self.check_is_explainer()
 
+        # TODO: Deal with Coalition of feature for categorical features in the importance selection
         if X.shape[1] > 10:
             feature_importance = -np.array(self.model.feature_importances_)
             search_space = list(np.argsort(feature_importance))
@@ -657,13 +877,88 @@ class ACXplainer:
         else:
             search_space = [i for i in range(X.shape[1])]
 
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+
         return cyext_acv.global_sdp_rf(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
                                        self.classifier, t, C,
                                        pi_level, minimal, stop, search_space[:10])
 
-    def importance_ddp_rf(self, X, y, data, y_data, min_node_size=5, t=20,
+    def importance_ddp_rf(self, X, y, data, y_data, min_node_size=5, t=None,
+                          C=[[]], pi_level=0.9, minimal=1, stop=True):
+        """
+         Estimate the Minimal Sufficient Explanations and the Global sdp importance of a set of samples X using the
+         consistent estimator (Projected Forest + Quantile Regression), see Paper [ref].
+
+        Args:
+            X (numpy.ndarray): A matrix of samples (# samples X # features) on which to compute the SDP
+
+            y (numpy.ndarray): 1-D array (# samples) the targets of X
+
+            data (numpy.ndarray): The background dataset that is used for the estimation of the SDP. It should be the
+                                training samples.
+
+            y_data (numpy.ndarray): The targets of the background dataset
+
+            min_node_size (int): The minimal node size
+
+            t (float): The level of variations around the prediction that defines the SDP in regression (only for regression)
+
+            C (list[list[int]]): A list that contains a list of the indices of column for each grouped variables
+
+            pi_level (float): The minimal value of the Same Decision Probability (SDP) of the Sufficient Explanations.
+                              It should be in (0, 1).
+
+            minimal (int): It will search the Sufficient Explanations from subsets of size "minimal" instead of 1 by default
+
+            stop (bool): If stop=True, it will stop searching for the Sufficient Explanations if it does not find
+                         any Sufficient Explanations smaller than (# features / 2), otherwise it will continues until
+                         end.
+
+        Returns:
+            global_sdp_importance (numpy.ndarray): A 1-D matrix (# features) that is the global explanatory importance
+                                                 based on samples X. For a given i, sdp_importance[i] corresponds to the
+                                                 frequency of apparition of feature i in the Minimal Sufficient Explanations
+                                                 of the set of samples X
+
+            sdp_index (numpy.ndarray): A matrix (# samples X # features) that contains the indices of the variables in the
+                                     the Minimal Sufficient Explanations for each sample. For a given i, the positive
+                                     values of sdp_index[i] corresponds to the Minimal Sufficient Explanations of
+                                     observation i.
+
+            size (numpy.ndarray): A 1-D matrix (# samples) that contains the size of the Minimal Sufficient Explanation
+                                for each sample.
+
+            sdp (numpy.ndarray): A 1-D matrix (# samples) that contains the Same Decision Probability (SDP)
+                               of the Sufficient Explanation for each sample.
+        """
+        X, y = check_X_y(X, y, dtype=np.double)
+        data, y_data = check_X_y(data, y_data, dtype=np.double)
+        y, y_data = as_float_array(y).astype(np.double), as_float_array(y_data).astype(np.double)
+        self.check_is_explainer()
+
+        # TODO: Deal with Coalition of feature for categorical features in the importance selection
+        if X.shape[1] > 10:
+            feature_importance = -np.array(self.model.feature_importances_)
+            search_space = list(np.argsort(feature_importance))
+            # flat_list = [item for t in self.ACXplainer.node_idx_trees for sublist in t for item in sublist]
+            # node_idx = pd.Series(flat_list)
+            # search_space = []
+            # for v in (node_idx.value_counts().keys()):
+            #     search_space += [v]
+        else:
+            search_space = [i for i in range(X.shape[1])]
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
+        return cyext_acv.global_ddp_rf(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
+                                       self.ACXplainer.children_left,
+                                       self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
+                                       self.classifier, t, C,
+                                       pi_level, minimal, stop, search_space[:10])
+
+    def importance_cdp_rf(self, X, y, down, up, data, y_data, min_node_size=5, t=None,
                           C=[[]], pi_level=0.9, minimal=1, stop=True):
         """
          Estimate the Minimal Sufficient Explanations and the Global sdp importance of a set of samples X using the
@@ -726,14 +1021,16 @@ class ACXplainer:
             #     search_space += [v]
         else:
             search_space = [i for i in range(X.shape[1])]
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
 
-        return cyext_acv.global_ddp_rf(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
+        return cyext_acv.global_cdp_rf(X, y, down, up, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
                                        self.classifier, t, C,
                                        pi_level, minimal, stop, search_space[:10])
 
-    def sufficient_expl_rf(self, X, y, data, y_data, min_node_size=5, t=20,
+    def sufficient_expl_rf(self, X, y, data, y_data, min_node_size=5, t=None,
                            C=[[]], pi_level=0.9, minimal=1, stop=True):
 
         """
@@ -790,13 +1087,16 @@ class ACXplainer:
         else:
             search_space = [i for i in range(X.shape[1])]
         # TODO: Remove the [-1] in the Sufficent Coal
+
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         return cyext_acv.sufficient_expl_rf(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
                                             self.ACXplainer.children_left,
                                             self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
                                             self.classifier, t, C,
                                             pi_level, minimal, stop, search_space[:10])
 
-    def sufficient_cxpl_rf(self, X, y, data, y_data, min_node_size=5, t=20,
+    def sufficient_cxpl_rf(self, X, y, data, y_data, min_node_size=5, t=None,
                            C=[[]], pi_level=0.9, minimal=1, stop=True):
 
         """
@@ -854,6 +1154,8 @@ class ACXplainer:
             search_space = [i for i in range(X.shape[1])]
         # TODO: Remove the [-1] in the Sufficent Coal
         # TODO: Check S_bar_set for this function
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         return cyext_acv.sufficient_cxpl_rf(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
                                             self.ACXplainer.children_left,
                                             self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
@@ -861,7 +1163,7 @@ class ACXplainer:
                                             pi_level, minimal, stop, search_space[:10])
 
 
-    def compute_exp_rf(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_exp_rf(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the Conditional Expectation (exp) of a set of samples X given subset S using the consistent estimator
          (Projected Forest), see Paper [ref]
@@ -896,14 +1198,15 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         exp = cyext_acv.compute_exp_rf(X, y, data, y_data, S, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
                                        self.classifier, t)
         return exp
 
-    def compute_cdf_rf(self, X, y, data, y_data, S, min_node_size=5, t=20):
+    def compute_cdf_rf(self, X, y, data, y_data, S, min_node_size=5, t=None):
         """
          Estimate the Projected Cumulative Distribution Function (P-CDF) of a set of samples X given subset S using the
           consistent estimator (Projected Forest + Quantile Regression), see Paper [ref]
@@ -938,7 +1241,8 @@ class ACXplainer:
             raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         sdp = cyext_acv.compute_cdf_rf(X, y, data, y_data, S, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, self.ACXplainer.max_depth, min_node_size,
@@ -1040,7 +1344,7 @@ class ACXplainer:
     #     return y_quantiles_diff
 
 
-    def compute_ddp_intv_weights(self, X, y, data, y_data, S, cond, min_node_size=5, t=20, pi_level=0.95,
+    def compute_ddp_intv_weights(self, X, y, data, y_data, S, cond, min_node_size=5, t=None, pi_level=0.95,
                              verbose=False, algo2=False, rdm_size=20):
         """
         Compute P(Y \neq y | X_{\bar{S}} in rule_{\bar{S}}), X_S is missing
@@ -1080,7 +1384,8 @@ class ACXplainer:
 
         self.check_is_explainer()
 
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         ddp, w = cyext_acv.compute_ddp_intv_weights(X, y, data, y_data, S,
                                                                            self.ACXplainer.features,
                                                                            self.ACXplainer.thresholds,
@@ -1092,7 +1397,7 @@ class ACXplainer:
 
         return ddp, w
 
-    def compute_ddp_intv_same(self, X, y, data, y_data, S, cond, min_node_size=5, t=20, pi_level=0.95,
+    def compute_ddp_intv_same(self, X, y, data, y_data, S, cond, min_node_size=5, t=None, pi_level=0.95,
                              verbose=False, algo2=False, rdm_size=20):
         """
          Compute P(Y \neq y | X_{\bar{S}} in rule_{\bar{S}}), X_S is missing using same S
@@ -1131,8 +1436,8 @@ class ACXplainer:
         #     raise ValueError('{} for X (samples) and S (coalition)'.format(exp))
 
         self.check_is_explainer()
-
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         ddp = cyext_acv.compute_ddp_intv_same_set(X, y, data, y_data, S,
                                                                            self.ACXplainer.features,
                                                                            self.ACXplainer.thresholds,
@@ -1145,7 +1450,7 @@ class ACXplainer:
         return ddp
 
 
-    def importance_ddp_intv(self, X, y, data, y_data, cond, min_node_size=5, t=20,
+    def importance_ddp_intv(self, X, y, data, y_data, cond, min_node_size=5, t=None,
                           C=[[]], pi_level=0.9, minimal=1, stop=True):
         """
          Find Minimal Divergent Set for Rule P(Y \neq y | X_{\bar{S}} in rule_{\bar{S}}), X_S is missing
@@ -1207,7 +1512,8 @@ class ACXplainer:
             #     search_space += [v]
         else:
             search_space = [i for i in range(X.shape[1])]
-
+        if not self.classifier and t is None:
+            t = self.model.predict_quantiles(X, quantiles=[0.05, 0.95])
         return cyext_acv.global_ddp_intv(X, y, data, y_data, self.ACXplainer.features, self.ACXplainer.thresholds,
                                        self.ACXplainer.children_left,
                                        self.ACXplainer.children_right, cond, self.ACXplainer.max_depth, min_node_size,
