@@ -1300,7 +1300,6 @@ def return_edges(list_rec):
 
 def mc_approx_rules(rules, rules_data, S_star, strategy='random', maxitr=1000, interiorOnly=False,
                     threshold=None, verbose=False):
-
     #     strategy = 'random' # or 'even' or 'sequential'
     #     maxitr = 1000 # how many queries to do since last best found before satisfied
     # whether to consider only rectangles that are bounded on all sides by points rather than limits of the space
@@ -1343,9 +1342,13 @@ def return_xy_cnt(x, y_target, S_bar_set, x_train, y_train, w):
         if wi != 0 and y_train[i] == y_target:
             x_train_cnt.append(x_train[i].copy())
 
-    x_train_cnt = np.array(x_train_cnt)
+    if len(x_train_cnt) == 0:
+        x_train_cnt = np.array(x).reshape(1, -1)
+        y_train_cnt = np.array(y_target).reshape(1, -1)
+    else:
+        x_train_cnt = np.array(x_train_cnt)
     #     x_train_cnt[:, S_bar_set] = x[S_bar_set]
-    y_train_cnt = np.array(x_train_cnt.shape[0] * [y_target])
+        y_train_cnt = np.array(x_train_cnt.shape[0] * [y_target])
 
     return x_train_cnt, y_train_cnt
 
@@ -1390,7 +1393,7 @@ def get_compatible_leaf(acvtree, x, y_target, S_star, S_bar_set, w, x_train, y_t
     """
     x_train_cnt, y_train_cnt = return_xy_cnt(x, y_target, S_bar_set, x_train, y_train, w)
 
-    compatible_leaves = return_leaf_cnt(acvtree, S_star, x_train_cnt, y_train_cnt, x_train, y_train)
+    compatible_leaves = return_leaf_cnt(acvtree, S_star, x_train_cnt, y_train_cnt, x_train, y_train, pi)
     compatible_leaves = np.unique(compatible_leaves, axis=0)
     compatible_leaves = remove_in(compatible_leaves)
     #     compatible_leaves = np.rint(compatible_leaves)
@@ -1509,7 +1512,8 @@ def return_ge_global_counterfactuals(ac_explainer, data, y_data, s_star, n_star,
                                                    x_train, y_train, pi=pi_level, acc_level=acc_level)
         suf_leaves.append(suf_leaf)
 
-        counterfactuals, counterfactuals_sdp, w_cond = return_ge_counterfactuals(ac_explainer, suf_leaf, s_star[i], n_star[i],
+        counterfactuals, counterfactuals_sdp, w_cond = return_ge_counterfactuals(ac_explainer, suf_leaf, s_star[i],
+                                                                                 n_star[i],
                                                                                  data[i].reshape(1, -1),
                                                                                  y_data[i].reshape(1, -1), x_train,
                                                                                  y_train,
@@ -1521,6 +1525,7 @@ def return_ge_global_counterfactuals(ac_explainer, data, y_data, s_star, n_star,
 
     return counterfactuals_samples, counterfactuals_samples_sdp, counterfactuals_samples_w
 
+
 def print_rule(col_name, r, decision=None, sdp=True, output=None):
     for i, col in enumerate(col_name):
         if not ((r[i, 0] <= -1e+10 and r[i, 1] >= 1e+10)):
@@ -1548,6 +1553,22 @@ def generate_candidate(x, S, x_train, cond, n_iterations):
 
 def simulated_annealing(outlier_score, x, S, x_train, cond, batch, max_iter, temp,
                         max_iter_convergence=10):
+    """
+    Generate sample s.t. (X | X_S \in cond) using simulated annealing and outlier score.
+    Args:
+        outlier_score (lambda functon): outlier_score(X) return a outlier score. If the value are negative, then the observation is an outlier.
+        x (numpy.ndarray)): 1-D array, an observation
+        S (list): contains the indices of the variables on which to condition
+        x_train (numpy.ndarray)): 2-D array represent the training samples
+        cond (numpy.ndarray)): 3-D (#variables x 2 x 1) representing the hyper-rectangle on which to condition
+        batch (int): number of sample by iteration
+        max_iter (int): number of iteration of the algorithm
+        temp (double): the temperature of the simulated annealing algorithm
+        max_iter_convergence (double): minimun number of iteration to stop the algorithm if it find an in-distribution observation
+
+    Returns:
+        The generated sample, and its outlier score
+    """
     best = generate_candidate(x, S, x_train, cond, 1)
     best_eval = outlier_score(best)[0]
     curr, curr_eval = best, best_eval
@@ -1580,8 +1601,10 @@ def simulated_annealing(outlier_score, x, S, x_train, cond, batch, max_iter, tem
 
     return best, best_eval
 
+
 dataset = 'none'
 
+
 def save_model(model, name='{}'.format(dataset)):
     with open('{}.pickle'.format(name), 'wb') as f:
         pickle.dump(model, f)
@@ -1632,11 +1655,11 @@ class RunExperiments:
         self.accuracy_regional_original = None
         self.coverage_regional = None
 
-    def run_local_divergent_set(self, x, y, stop=True, pi_level=0.8):
+    def run_local_divergent_set(self, x, y, t=20, stop=True, pi_level=0.8):
         print('### Computing the local divergent set of (x, y)')
 
         self.ddp_importance_local, self.ddp_index_local, self.size_local, self.ddp_local = \
-            self.acv_explainer.importance_ddp_rf(x, y, self.x_train, self.y_train,
+            self.acv_explainer.importance_ddp_rf(x, y, self.x_train, self.y_train, t=t,
                                                  stop=stop, pi_level=pi_level)
 
         self.S_star_local, self.S_bar_set_local = \
@@ -1652,7 +1675,7 @@ class RunExperiments:
                                                                              self.S_star_local, self.S_bar_set_local,
                                                                              self.x_train, self.y_train, self.w_local,
                                                                              acc_level=acc_level,
-                                                                             pi_level=acc_level)
+                                                                             pi_level=pi_level)
 
     def run_sampling_local_counterfactuals(self, x, y, batch=1000, max_iter=1000, temp=0.5):
         print('### Sampling using the local counterfactual rules of (x, y)')
@@ -1835,4 +1858,4 @@ class RunExperiments:
                 print_rule(self.columns_name, self.counterfactuals_samples_local[idt][l][0],
                            self.counterfactuals_samples_sdp_local[idt][l], False, False)
                 print('  ')
-            print(' ')
\ No newline at end of file
+            print(' ')
